library ieee;
library work;
use work.Latches_Flipflops.all;
use ieee.std_logic_1164.all;

entity COMMAND_MODULE is
  
  port (

-- Chip Scope Pro Logic Analyzer control -- bgb

    CSP_VME_LA_CTRL: inout std_logic_vector(35 downto 0); -- bgb
    VME_LA_TRG : inout std_logic; -- bgb
	 
    FASTCLK: in std_logic;
    SLOWCLK: in std_logic;

    GA : in std_logic_vector(5 downto 0);
    ADR : in std_logic_vector(23 downto 1);
    AM : in std_logic_vector(5 downto 0);

    AS : in std_logic;
    DS0 : in std_logic;
    DS1 : in std_logic;
    LWORD : in std_logic;
    WRITER : in std_logic; --NOTE: this is the only signal whose name was changed
    IACK : in std_logic;
    BERR : in std_logic;
    SYSFAIL : in std_logic;
    DTACK_B : in std_logic; -- bgb added for logic anaylzer
    
    DEVICE   : inout std_logic_vector(9 downto 0);
    STROBE   : inout std_logic;
    COMMAND  : inout std_logic_vector(9 downto 0);
    ADRS     : out std_logic_vector(17 downto 2);  --NOTE: output of ADRS
 
-- bgb   INDATA   : out std_logic_vector(15 downto 0);
-- bgb   OUTDATA : in std_logic_vector(15 downto 0);

	 VME_DATA : in std_logic_vector(15 downto 0);
    TOVME_B    : out std_logic;
    DOE_B    : out std_logic;
-- bgb (not used)    VMEREADY : out std_logic;
	 
    DIAGOUT  : out std_logic_vector(19 downto 0);
    LED      : out std_logic_vector(2 downto 0)
	 
    );
    
end COMMAND_MODULE;

architecture COMMAND_MODULE_Arch of COMMAND_MODULE is


-- bgb 
-- bgb  Added Chip Scope Pro Logic analyzer for VME signals
-- bgb 

component csp_vme_la
  PORT (
    CONTROL : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    CLK : IN STD_LOGIC;
    DATA : IN STD_LOGIC_VECTOR(141 DOWNTO 0);
    TRIG0 : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    TRIG_OUT : OUT STD_LOGIC
	 );

end component;

-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of csp_vme_la: component is true;

-- bgb 
-- bgb End of Logic Analyzer declaration
-- bgb 

  --Declaring internal signals
  signal CGA : std_logic_vector(5 downto 0);  --NOTE: replacing CGAP with CGA(5)
  signal AMS : std_logic_vector(5 downto 0);
  SIGNAL ADRS_INNER : std_logic_vector(23 downto 1);
  signal GOODAM : std_logic;
  signal VALIDAM : std_logic;
  signal VALIDGA : std_logic;
  signal SYSOK : std_logic;
  signal OLDCRATE : std_logic;
  signal PRE_BOARDENB : std_logic;
  signal BROADCAST : std_logic;
  signal BOARDENB : std_logic;
  signal BOARD_SEL_NEW : std_logic;
  signal ASYNSTRB : std_logic;
  signal ASYNSTRB_NOT : std_logic;
  signal FASTCLK_NOT : std_logic;
  signal STROBE_TEMP1 : std_logic;
  signal STROBE_TEMP2 : std_logic;
  signal ADRSHIGH : std_logic;
  signal D1,C1,Q1,D2,C2,Q2,D3,C3,Q3,D4,C4 : std_logic;
  signal D1_second,C1_second,Q1_second,D2_second,C2_second,Q2_second,D3_second,C3_second,Q3_second,D4_second,C4_second,Q4_second,D5_second,C5_second : std_logic;
  signal TOVME_INNER : std_logic;
--  signal DATA_INNER : std_logic_vector(15 downto 0);

  signal CE_DOE_B, CLR_DOE_B : std_logic;
  signal TIMER : std_logic_vector(7 downto 0);
  signal TOVME_INNER_B : std_logic;
  signal blank1, blank2 : std_logic;
 

  --Declaring Logic Analyzer signals -- bgb
  signal vme_la_data : std_logic_vector(141 downto 0);
  signal vme_la_trig : std_logic_vector(7 downto 0);
  
  -----------------------------------------------------------------------------

begin  --Architecture

-- bgb 
-- bgb  Added Chip Scope Pro Logic analyzer for VME signals
-- bgb 
csp_vme_la_i : csp_vme_la
  port map (
    CONTROL => CSP_VME_LA_CTRL,
    CLK => FASTCLK,
    DATA => vme_la_data,
    TRIG0 => vme_la_trig,
    TRIG_OUT => VME_LA_TRG
	 );
	 
-- trigger assignments (8 bits)
  vme_la_trig(0) <= AS;
  vme_la_trig(1) <= DS0;
  vme_la_trig(2) <= DS1;
  vme_la_trig(3) <= LWORD;
  vme_la_trig(4) <= WRITER;
  vme_la_trig(5) <= BERR;
  vme_la_trig(6) <= DEVICE(1);
  vme_la_trig(7) <= STROBE;
	 
-- data assignments (128 bits)
  vme_la_data(5 downto 0)   <= GA;
  vme_la_data(11 downto 6)  <= CGA;
  vme_la_data(35 downto 12) <= ADR & '0';
  vme_la_data(59 downto 36) <= ADRS_INNER & '0';
  vme_la_data(65 downto 60) <= AM;
  vme_la_data(71 downto 66) <= AMS;
  vme_la_data(87 downto 72) <= VME_DATA;
  vme_la_data(97 downto 88) <= DEVICE;
  vme_la_data(107 downto 98) <= COMMAND;
  vme_la_data(115 downto 108) <= TIMER;
  vme_la_data(116) <= AS;
  vme_la_data(117) <= DS0;
  vme_la_data(118) <= DS1;
  vme_la_data(119) <= LWORD;
  vme_la_data(120) <= WRITER;
  vme_la_data(121) <= DTACK_B;
  vme_la_data(122) <= BERR;
  vme_la_data(123) <= STROBE_TEMP1;
  vme_la_data(124) <= STROBE_TEMP2;
  vme_la_data(125) <= STROBE;
  vme_la_data(126) <= TOVME_INNER_B;
  vme_la_data(127) <= GOODAM;
  vme_la_data(128) <= VALIDGA;
  vme_la_data(129) <= SYSOK;
  vme_la_data(130) <= OLDCRATE;
  vme_la_data(131) <= PRE_BOARDENB;
  vme_la_data(132) <= BROADCAST;
  vme_la_data(133) <= BOARDENB;
  vme_la_data(134) <= BOARD_SEL_NEW;
  vme_la_data(135) <= ASYNSTRB;
  vme_la_data(136) <= ADRSHIGH;
  vme_la_data(137) <= CE_DOE_B;
  vme_la_data(138) <= CLR_DOE_B;
  vme_la_data(139) <= TIMER(7); --DOE_B;
  vme_la_data(140) <= '0';
  vme_la_data(141) <= VME_LA_TRG; 
  
-- bgb 
-- bgb End of Logic Analyzer instantiation
-- bgb 


  --All processes will be called CREATE_{name of signal they create}
  --If a process creates more than one signal, one name will be used and then
  --the other possible names will be in the comments
  --This is so the reader can use ctrl+f functions to find relevant processes

  CREATE_DOE_B: process (TOVME_INNER_B, TIMER, SLOWCLK) 
  begin
    if (TOVME_INNER_B='0' and TIMER(7)='0') then
      CE_DOE_B <= '1';
    else
      CE_DOE_B <= '0';
    end if;
    
    CLR_DOE_B <= TOVME_INNER_B;
 
    CB8CE(SLOWCLK,CE_DOE_B,CLR_DOE_B,TIMER,TIMER,blank1,blank2);

	 DOE_B <= TIMER(7);
	 
  end process;
  
  CREATE_VALIDGA: process (GA,CGA)
  begin  
    CGA <= not GA;
    VALIDGA <= CGA(0) xor CGA(1) xor CGA(2) xor CGA(3) xor CGA(4) xor CGA(5);
  end process;

  CREATE_OLDCRATE: process (CGA)
  begin  
    if (CGA="000000") then
      OLDCRATE <= '1';
    else
      OLDCRATE <= '0';
    end if;
  end process;

  CREATE_AMS: process (AM,AS)
  begin  
    ILD6(AM(5 downto 0),AS,AMS(5 downto 0));  -- address modifier latched with strobe
  end process;

  CREATE_VALIDAM: process (AMS,LWORD)   --Also could be named CREATE_GOODAM
  begin  
	 if(LWORD='1' and (("00" & AMS) = x"39" or ("00" & AMS) = x"3A" or ("00" & AMS) = x"3D" or ("00" & AMS) = x"3E")) then -- non-priv/supervisory prog/data access
      GOODAM <= '1';
      VALIDAM <= '1';
    else
      GOODAM <= '0';
      VALIDAM <= '0';
    end if;
  end process;
  
CREATE_FASTCLK_NOT: process (FASTCLK)
  begin
    FASTCLK_NOT <= not FASTCLK;
  end process;
  
  CREATE_STROBE: process (ADR,CGA,GOODAM,WRITER,VALIDGA,OLDCRATE,BROADCAST,FASTCLK,FASTCLK_NOT,ADRS_INNER, BOARD_SEL_NEW)    --Also CREATE_LED0, CREATE_LED2
  begin  
    ILD6(ADR(23 downto 18),AS,ADRS_INNER(23 downto 18));
    
    if (ADRS_INNER(23)=CGA(4) and ADRS_INNER(22)=CGA(3) and ADRS_INNER(21)=CGA(2) and ADRS_INNER(20)=CGA(1) and ADRS_INNER(19)=CGA(0)) then
      BOARD_SEL_NEW <= '1';
    else
      BOARD_SEL_NEW <= '0';
    end if;

    if (BOARD_SEL_NEW='1' and VALIDGA='1') then
      PRE_BOARDENB <= '1';
    else
      PRE_BOARDENB <= '0';
    end if;

    if (ADRS_INNER(21)='0' and ADRS_INNER(19)='1' and ADRS_INNER(22)='1' and ADRS_INNER(23)='1') then
      BROADCAST <= '1';
    else
      BROADCAST <= '0';
    end if;

    if (OLDCRATE='1' or PRE_BOARDENB='1' or BROADCAST='1') then
      BOARDENB <= '1';
    else
      BOARDENB <= '0';
    end if;
    
    if (SYSFAIL='1' and IACK='1') then
      SYSOK <= '1';
    else
      SYSOK <= '0';
    end if;

    if (GOODAM='1' and WRITER='1' and SYSOK='1' and BOARDENB='1') then
      TOVME_INNER_B <= '0';
      TOVME_B <= '0';
      LED(0) <= '1';
    else
      TOVME_INNER_B <= '1';
      TOVME_B <= '1';
      LED(0) <= '0';
    end if;

    if (SYSOK='1' and VALIDAM='1' and BOARDENB='1' and DS0='0' and DS1='0') then
      ASYNSTRB <= '1';
    else
      ASYNSTRB <= '0';
    end if;
    
    ASYNSTRB_NOT <= not ASYNSTRB;
    FDC(ASYNSTRB,FASTCLK,ASYNSTRB_NOT,STROBE_TEMP1);
    FDC_1(ASYNSTRB,FASTCLK,ASYNSTRB_NOT,STROBE_TEMP2);

    if (STROBE_TEMP1='1' and STROBE_TEMP2='1') then
      STROBE <= '1';
      LED(2) <= '0';
    else
      STROBE <= '0';
      LED(2) <= '1';
    end if;

  end process;

  CREATE_LED1: process (ASYNSTRB)
  begin
    LED(1) <= not ASYNSTRB;
  end process;
 
  
  CREATE_DIAGOUT: process (ADRS_INNER(23 downto 19),CGA,VALIDGA,SYSOK,VALIDAM,BOARD_SEL_NEW,DS0,ASYNSTRB)
  begin  
    DIAGOUT(0) <= ADRS_INNER(23);
    DIAGOUT(1) <= CGA(4);
    DIAGOUT(2) <= ADRS_INNER(21);
    DIAGOUT(3) <= CGA(2);
    DIAGOUT(4) <= ADRS_INNER(22);
    DIAGOUT(5) <= CGA(3);
    DIAGOUT(6) <= ADRS_INNER(20);
    DIAGOUT(7) <= CGA(1);
    DIAGOUT(8) <= ADRS_INNER(19);
    DIAGOUT(9) <= CGA(0);
    DIAGOUT(10) <= CGA(5);
    DIAGOUT(11) <= VALIDGA;
    DIAGOUT(12) <= SYSOK;
    DIAGOUT(13) <= VALIDAM;
    DIAGOUT(14) <= BOARD_SEL_NEW;
    DIAGOUT(15) <= DS0;
    DIAGOUT(16) <= ASYNSTRB;
    DIAGOUT(17) <= STROBE_TEMP1;
    DIAGOUT(18) <= STROBE_TEMP2;
    DIAGOUT(19) <= FASTCLK_NOT;
  end process;

  CREATE_COMMAND: process (ADRS_INNER(11 downto 2))
  begin  
    COMMAND(0) <= ADRS_INNER(2);
    COMMAND(1) <= ADRS_INNER(3);
    COMMAND(2) <= ADRS_INNER(4);
    COMMAND(3) <= ADRS_INNER(5);
    COMMAND(4) <= ADRS_INNER(6);
    COMMAND(5) <= ADRS_INNER(7);
    COMMAND(6) <= ADRS_INNER(8);
    COMMAND(7) <= ADRS_INNER(9);
    COMMAND(8) <= ADRS_INNER(10);
    COMMAND(9) <= ADRS_INNER(11);
  end process;


  CREATE_ADRS_INNER_13_18: process (ADR(18 downto 13),AS)
  begin
    ILD6(ADR(18 downto 13),AS,ADRS_INNER(18 downto 13));
  end process;
  
  CREATE_ADRS_INNER_7_12: process (ADR(12 downto 7),AS)
  begin
    ILD6(ADR(12 downto 7),AS,ADRS_INNER(12 downto 7));
  end process;
  
  CREATE_ADRS_INNER_2_6: process (ADR(6 downto 1),AS)
  begin
    ILD6(ADR(6 downto 1),AS,ADRS_INNER(6 downto 1));
  end process;

  CREATE_ADRS: process (ADRS_INNER)
  begin
    ADRS <= ADRS_INNER(17 downto 2);
  end process;

  
  
  CREATE_DEVICE: process (ADRS_INNER(18 downto 12))
  begin
    if (ADRS_INNER(18)='1' or ADRS_INNER(17)='1' or ADRS_INNER(16)='1') then
      ADRSHIGH <= '1';
    else
      ADRSHIGH <= '0';
    end if;

    if (ADRS_INNER(12)='0' and ADRS_INNER(13)='0' and ADRS_INNER(14)='0' and ADRS_INNER(15)='0' and ADRSHIGH='0') then
      DEVICE(0) <= '1';
    else
      DEVICE(0) <= '0';
    end if;

    if (ADRS_INNER(12)='1' and ADRS_INNER(13)='0' and ADRS_INNER(14)='0' and ADRS_INNER(15)='0' and ADRSHIGH='0') then
      DEVICE(1) <= '1';
    else
      DEVICE(1) <= '0';
    end if;

    if (ADRS_INNER(12)='0' and ADRS_INNER(13)='1' and ADRS_INNER(14)='0' and ADRS_INNER(15)='0' and ADRSHIGH='0') then
      DEVICE(2) <= '1';
    else
      DEVICE(2) <= '0';
    end if;

    if (ADRS_INNER(12)='1' and ADRS_INNER(13)='1' and ADRS_INNER(14)='0' and ADRS_INNER(15)='0' and ADRSHIGH='0') then
      DEVICE(3) <= '1';
    else
      DEVICE(3) <= '0';
    end if;

    if (ADRS_INNER(12)='0' and ADRS_INNER(13)='0' and ADRS_INNER(14)='1' and ADRS_INNER(15)='0' and ADRSHIGH='0') then
      DEVICE(4) <= '1';
    else
      DEVICE(4) <= '0';
    end if;

    if (ADRS_INNER(12)='1' and ADRS_INNER(13)='0' and ADRS_INNER(14)='1' and ADRS_INNER(15)='0' and ADRSHIGH='0') then
      DEVICE(5) <= '1';
    else
      DEVICE(5) <= '0';
    end if;

    if (ADRS_INNER(12)='0' and ADRS_INNER(13)='1' and ADRS_INNER(14)='1' and ADRS_INNER(15)='0' and ADRSHIGH='0') then
      DEVICE(6) <= '1';
    else
      DEVICE(6) <= '0';
    end if;

    if (ADRS_INNER(12)='1' and ADRS_INNER(13)='1' and ADRS_INNER(14)='1' and ADRS_INNER(15)='0' and ADRSHIGH='0') then
      DEVICE(7) <= '1';
    else
      DEVICE(7) <= '0';
    end if;

    if (ADRS_INNER(12)='0' and ADRS_INNER(13)='0' and ADRS_INNER(14)='0' and ADRS_INNER(15)='1' and ADRSHIGH='0') then
      DEVICE(8) <= '1';
    else
      DEVICE(8) <= '0';
    end if;

    if (ADRS_INNER(12)='1' and ADRS_INNER(13)='0' and ADRS_INNER(14)='0' and ADRS_INNER(15)='1' and ADRSHIGH='0') then
      DEVICE(9) <= '1';
    else
      DEVICE(9) <= '0';
    end if;
    
  end process;

-- bgb
-- This function is in the IOB
--
--  CREATE_INDATA: process (OUTDATA,DATA,TOVME_INNER_B)
--  begin
--    DATA_INNER <= DATA;
--    if (TOVME_INNER_B='0') then
--      DATA_INNER <= OUTDATA;
--    end if;
--    INDATA <= DATA_INNER;
--  end process;
--
  

end COMMAND_MODULE_Arch;
